{% load i18n %}

<form action="https://secure.payu.ro/order/lu.php" method="post">
    {{ form.as_p }}

    <button type="submit" class="btn btn-primary pay">
        {% trans "Pay by Card" %}
    </button>
</form>
